# Generated by cucumber-sinatra. (Tue Jul 27 10:45:23 -0400 2010)

require File.join(File.dirname(__FILE__), '..', '..', 'lib/example_sinatra_app.rb')
require 'capybara'
require 'capybara/cucumber'
require 'rspec/core'
#require 'activesupport'
require File.expand_path(File.dirname(__FILE__) + '../../../../../lib/email_spec')
require 'email_spec/cucumber'

ExampleSinatraApp.set(:environment, :test)

Capybara.app = ExampleSinatraApp

class ExampleSinatraAppWorld
  include Capybara::DSL
end

World do
  ExampleSinatraAppWorld.new
end
