# Be sure to restart your server when you modify this file.

# Your secret key for verifying the integrity of signed cookies.
# If you change this key, all old signed cookies will become invalid!
# Make sure the secret is at least 30 characters and all random, 
# no regular words or you'll be exposed to dictionary attacks.
Rails.application.config.secret_token = 'c63e0079364c42f86ef0256e8eb5e8f6993b30f1e64e4374f5211ddf4efdcd0895282c6383bd58dc817716b7ecc15f52de122e2df27fa326266d4163e4e5fe7d'
